use std::cmp::Ordering;

use grid::Grid;

pub fn start() {
	let grid = generate_grid();

	let mut visible_trees: usize = (grid.rows() + grid.cols()) * 2 - 4;

	for row in 1..grid.rows() - 1 {
		for column in 1..grid.cols() - 1 {
			if check_direction(&grid, row, column)
				.iter()
				.map(|f| f.iter().any(|f| f >= &grid[row][column]))
				.any(|t| t == false)
			{
				visible_trees += 1
			}
		}
	}

	let mut products: Vec<usize> = vec![];

	for row in 0..grid.rows() {
		for column in 0..grid.cols() {
			let target = &grid[row][column];

			let mut i = 0;
			let test: Vec<usize> = check_direction(&grid, row, column)
				.iter()
				.map(|val| {
					let mut counter = 0;
					let mut numbers = val.clone();

					if i == 0 || i == 2 {
						numbers = val.iter().rev().copied().collect();
					}

					for nr in &numbers {
						match nr.cmp(target) {
							Ordering::Less => counter += 1,
							Ordering::Equal | Ordering::Greater => {
								counter += 1;
								break;
							}
						}
					}
					i += 1;
					counter
				})
				.collect();

			products.push(test.iter().product());
		}
	}
	let res = *products.iter().max().unwrap();

	println!("Part 1: {}", visible_trees);
	println!("Part 2: {}", res);
}

fn generate_grid() -> Grid<u32> {
	let input = include_str!("input.txt").replace("\r", "");
	let mut grid: Grid<u32> = Grid::new(0, 0);
	input
		.lines()
		.map(|f| f.chars().map(|c| c.to_digit(10).unwrap()).collect())
		.for_each(|row| grid.push_row(row));

	grid
}

fn check_direction(grid: &Grid<u32>, row: usize, column: usize) -> Vec<Vec<u32>> {
	let left = grid[row][..column].to_vec();
	let right = grid[row][column + 1..].to_vec();
	let up = grid.iter_col(column).take(row).cloned().collect();
	let down = grid.iter_col(column).skip(row + 1).cloned().collect();

	vec![left, right, up, down]
}
